// Generated from https://suulnnka.github.io/BullshitGenerator/index.html?%E4%B8%BB%E9%A2%98=%E7%A7%91%E5%AD%A6%E5%92%8C%E4%BA%BA%E6%96%87%E8%B0%81%E6%9B%B4%E6%9C%89%E6%84%8F%E4%B9%89&%E9%9A%8F%E6%9C%BA%E7%A7%8D%E5%AD%90=4255070717

var LARGE_TEXT_ZH = `维龙说过一句富有哲理的话，要成功不需要什么特别的才能，只要把你能做的小事做得好就行了。带着这句话，我们还要更加慎重的审视这个问题： 带着这些问题，我们来审视一下科学和人文谁更有意义。 既然如何， 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 卡耐基在不经意间这样说过，我们若已接受最坏的，就再没有什么损失。带着这句话，我们还要更加慎重的审视这个问题： 我认为， 既然如何。
　　歌德曾经说过，没有人事先了解自己到底有多大的力量，直到他试过以后才知道。这似乎解答了我的疑惑。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 从这个角度来看， 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 每个人都不得不面对这些问题。 在面对这种问题时， 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 每个人都不得不面对这些问题。 在面对这种问题时， 我们不得不面对一个非常尴尬的事实，那就是， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 总结的来说， 既然如何， 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 问题的关键究竟为何？ 科学和人文谁更有意义因何而发生？ 从这个角度来看， 我们不得不面对一个非常尴尬的事实，那就是， 卢梭曾经提到过，浪费时间是一桩大罪过。带着这句话，我们还要更加慎重的审视这个问题： 科学和人文谁更有意义，到底应该如何实现。 我们不得不面对一个非常尴尬的事实，那就是， 从这个角度来看， 拿破仑·希尔曾经提到过，不要等待，时机永远不会恰到好处。这启发了我， 一般来说， 在这种困难的抉择下，本人思来想去，寝食难安。 每个人都不得不面对这些问题。 在面对这种问题时， 老子在不经意间这样说过，知人者智，自知者明。胜人者有力，自胜者强。我希望诸位也能好好地体会这句话。 卡耐基在不经意间这样说过，我们若已接受最坏的，就再没有什么损失。带着这句话，我们还要更加慎重的审视这个问题。
　　歌德曾经提到过，没有人事先了解自己到底有多大的力量，直到他试过以后才知道。我希望诸位也能好好地体会这句话。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 伏尔泰在不经意间这样说过，坚持意志伟大的事业需要始终不渝的精神。这句话语虽然很短，但令我浮想联翩。 既然如何， 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 卡耐基在不经意间这样说过，我们若已接受最坏的，就再没有什么损失。我希望诸位也能好好地体会这句话。 我认为， 我们都知道，只要有意义，那么就必须慎重考虑。 我们不得不面对一个非常尴尬的事实，那就是， 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 科学和人文谁更有意义，到底应该如何实现。 德谟克利特在不经意间这样说过，节制使快乐增加并使享受加强。这似乎解答了我的疑惑。 我认为， 那么， 米歇潘曾经说过，生命是一条艰险的峡谷，只有勇敢的人才能通过。这句话语虽然很短，但令我浮想联翩。 一般来讲，我们都必须务必慎重的考虑考虑。 问题的关键究竟为何？ 总结的来说， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 总结的来说， 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 总结的来说， 我们都知道，只要有意义，那么就必须慎重考虑。 一般来说， 那么， 既然如此， 这样看来， 一般来讲，我们都必须务必慎重的考虑考虑。 一般来讲，我们都必须务必慎重的考虑考虑。 经过上述讨论， 一般来讲，我们都必须务必慎重的考虑考虑。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。
　　科学和人文谁更有意义因何而发生？ 一般来讲，我们都必须务必慎重的考虑考虑。 亚伯拉罕·林肯在不经意间这样说过，你活了多少岁不算什么，重要的是你是如何度过这些岁月的。带着这句话，我们还要更加慎重的审视这个问题： 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 总结的来说， 雷锋说过一句富有哲理的话，自己活着，就是为了使别人过得更美好。这句话语虽然很短，但令我浮想联翩。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 那么， 那么， 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 科学和人文谁更有意义因何而发生？ 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 冯学峰曾经说过，当一个人用工作去迎接光明，光明很快就会来照耀着他。这不禁令我深思。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 一般来讲，我们都必须务必慎重的考虑考虑。
　　每个人都不得不面对这些问题。 在面对这种问题时， 拿破仑·希尔曾经说过，不要等待，时机永远不会恰到好处。这句话语虽然很短，但令我浮想联翩。 我们不得不面对一个非常尴尬的事实，那就是， 从这个角度来看， 富兰克林曾经说过，你热爱生命吗？那么别浪费时间，因为时间是组成生命的材料。带着这句话，我们还要更加慎重的审视这个问题： 从这个角度来看， 而这些并不是完全重要，更加重要的问题是， 带着这些问题，我们来审视一下科学和人文谁更有意义。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 培根说过一句富有哲理的话，阅读使人充实，会谈使人敏捷，写作使人精确。这启发了我， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 我认为， 科学和人文谁更有意义，到底应该如何实现。 我认为， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 俾斯麦曾经说过，对于不屈不挠的人来说，没有失败这回事。这启发了我， 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 一般来说， 吉姆·罗恩说过一句富有哲理的话，要么你主宰生活，要么你被生活主宰。我希望诸位也能好好地体会这句话。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 富勒曾经说过，苦难磨炼一些人，也毁灭另一些人。这启发了我， 普列姆昌德曾经提到过，希望的灯一旦熄灭，生活刹那间变成了一片黑暗。带着这句话，我们还要更加慎重的审视这个问题： 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 奥斯特洛夫斯基曾经提到过，共同的事业，共同的斗争，可以使人们产生忍受一切的力量。　这启发了我， 科学和人文谁更有意义因何而发生？ 而这些并不是完全重要，更加重要的问题是， 我们都知道，只要有意义，那么就必须慎重考虑。 培根说过一句富有哲理的话，要知道对好事的称颂过于夸大，也会招来人们的反感轻蔑和嫉妒。这似乎解答了我的疑惑。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 那么， 总结的来说， 经过上述讨论， 一般来说， 问题的关键究竟为何？ 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 经过上述讨论， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 科学和人文谁更有意义因何而发生？ 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 科学和人文谁更有意义，到底应该如何实现。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。
　　郭沫若在不经意间这样说过，形成天才的决定因素应该是勤奋。这不禁令我深思。 歌德在不经意间这样说过，意志坚强的人能把世界放在手中像泥块一样任意揉捏。我希望诸位也能好好地体会这句话。 总结的来说， 一般来讲，我们都必须务必慎重的考虑考虑。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 既然如此， 洛克曾经提到过，学到很多东西的诀窍，就是一下子不要学很多。我希望诸位也能好好地体会这句话。 那么， 既然如此， 总结的来说， 一般来讲，我们都必须务必慎重的考虑考虑。 我们都知道，只要有意义，那么就必须慎重考虑。 这样看来， 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 培根说过一句富有哲理的话，合理安排时间，就等于节约时间。这启发了我， 爱迪生曾经说过，失败也是我需要的，它和成功对我一样有价值。我希望诸位也能好好地体会这句话。 海贝尔说过一句富有哲理的话，人生就是学校。在那里，与其说好的教师是幸福，不如说好的教师是不幸。这启发了我， 我们都知道，只要有意义，那么就必须慎重考虑。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 我们都知道，只要有意义，那么就必须慎重考虑。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。
　　科学和人文谁更有意义因何而发生？ 笛卡儿曾经提到过，读一切好书，就是和许多高尚的人谈话。这不禁令我深思。 问题的关键究竟为何？ 一般来讲，我们都必须务必慎重的考虑考虑。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 既然如何， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 这样看来， 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 奥斯特洛夫斯基曾经说过，共同的事业，共同的斗争，可以使人们产生忍受一切的力量。　我希望诸位也能好好地体会这句话。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。
　　塞内加曾经说过，勇气通往天堂，怯懦通往地狱。带着这句话，我们还要更加慎重的审视这个问题： 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 佚名曾经提到过，感激每一个新的挑战，因为它会锻造你的意志和品格。这启发了我， 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 经过上述讨论， 经过上述讨论。
　　本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 王阳明曾经说过，故立志者，为学之心也；为学者，立志之事也。这句话语虽然很短，但令我浮想联翩。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 带着这些问题，我们来审视一下科学和人文谁更有意义。 问题的关键究竟为何？ 科学和人文谁更有意义因何而发生？ 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 经过上述讨论， 一般来说， 迈克尔·F·斯特利说过一句富有哲理的话，最具挑战性的挑战莫过于提升自我。这启发了我， 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 从这个角度来看， 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 一般来讲，我们都必须务必慎重的考虑考虑。 笛卡儿曾经提到过，读一切好书，就是和许多高尚的人谈话。我希望诸位也能好好地体会这句话。 我们都知道，只要有意义，那么就必须慎重考虑。 我认为， 从这个角度来看， 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 那么， 这样看来， 普列姆昌德在不经意间这样说过，希望的灯一旦熄灭，生活刹那间变成了一片黑暗。这不禁令我深思。 一般来讲，我们都必须务必慎重的考虑考虑。 裴斯泰洛齐曾经提到过，今天应做的事没有做，明天再早也是耽误了。这似乎解答了我的疑惑。 佚名曾经说过，感激每一个新的挑战，因为它会锻造你的意志和品格。我希望诸位也能好好地体会这句话。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 从这个角度来看， 一般来说， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 从这个角度来看， 在这种困难的抉择下，本人思来想去，寝食难安。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 科学和人文谁更有意义因何而发生？ 从这个角度来看， 每个人都不得不面对这些问题。 在面对这种问题时， 那么， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 在这种困难的抉择下，本人思来想去，寝食难安。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 莎士比亚曾经说过，意志命运往往背道而驰，决心到最后会全部推倒。这不禁令我深思。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 经过上述讨论， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 洛克说过一句富有哲理的话，学到很多东西的诀窍，就是一下子不要学很多。这启发了我， 科学和人文谁更有意义，发生了会如何，不发生又会如何。
　　所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 而这些并不是完全重要，更加重要的问题是， 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 爱尔兰在不经意间这样说过，越是无能的人，越喜欢挑剔别人的错儿。这句话语虽然很短，但令我浮想联翩。 总结的来说， 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 一般来说， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 从这个角度来看， 我们不得不面对一个非常尴尬的事实，那就是， 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 亚伯拉罕·林肯说过一句富有哲理的话，你活了多少岁不算什么，重要的是你是如何度过这些岁月的。这句话语虽然很短，但令我浮想联翩。 每个人都不得不面对这些问题。 在面对这种问题时， 我们不得不面对一个非常尴尬的事实，那就是， 既然如此， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。
　　就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 在这种困难的抉择下，本人思来想去，寝食难安。 每个人都不得不面对这些问题。 在面对这种问题时， 贝多芬说过一句富有哲理的话，卓越的人一大优点是：在不利与艰难的遭遇里百折不饶。我希望诸位也能好好地体会这句话。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 赫尔普斯曾经提到过，有时候读书是一种巧妙地避开思考的方法。这启发了我， 裴斯泰洛齐在不经意间这样说过，今天应做的事没有做，明天再早也是耽误了。这句话语虽然很短，但令我浮想联翩。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 每个人都不得不面对这些问题。 在面对这种问题时， 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 博说过一句富有哲理的话，一次失败，只是证明我们成功的决心还够坚强。 维这不禁令我深思。 一般来讲，我们都必须务必慎重的考虑考虑。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 韩非在不经意间这样说过，内外相应，言行相称。我希望诸位也能好好地体会这句话。 科学和人文谁更有意义因何而发生？ 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 莎士比亚在不经意间这样说过，那脑袋里的智慧，就像打火石里的火花一样，不去打它是不肯出来的。这句话语虽然很短，但令我浮想联翩。 亚伯拉罕·林肯曾经提到过，我这个人走得很慢，但是我从不后退。这句话语虽然很短，但令我浮想联翩。
　　对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 总结的来说， 经过上述讨论， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 布尔沃曾经提到过，要掌握书，莫被书掌握；要为生而读，莫为读而生。这似乎解答了我的疑惑。 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 富兰克林曾经提到过，你热爱生命吗？那么别浪费时间，因为时间是组成生命的材料。这启发了我。
　　带着这些问题，我们来审视一下科学和人文谁更有意义。 一般来讲，我们都必须务必慎重的考虑考虑。 在这种困难的抉择下，本人思来想去，寝食难安。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 既然如何， 一般来说， 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 一般来讲，我们都必须务必慎重的考虑考虑。 我们不得不面对一个非常尴尬的事实，那就是， 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 每个人都不得不面对这些问题。 在面对这种问题时， 这样看来， 达·芬奇在不经意间这样说过，大胆和坚定的决心能够抵得上武器的精良。这启发了我， 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 既然如何， 俾斯麦说过一句富有哲理的话，失败是坚忍的最后考验。这句话语虽然很短，但令我浮想联翩。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 拿破仑·希尔在不经意间这样说过，不要等待，时机永远不会恰到好处。这句话语虽然很短，但令我浮想联翩。 问题的关键究竟为何？ 问题的关键究竟为何？ 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 科学和人文谁更有意义，到底应该如何实现。
　　要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 非洲在不经意间这样说过，最灵繁的人也看不见自己的背脊。这句话语虽然很短，但令我浮想联翩。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 歌德曾经说过，没有人事先了解自己到底有多大的力量，直到他试过以后才知道。这不禁令我深思。 贝多芬曾经说过，卓越的人一大优点是：在不利与艰难的遭遇里百折不饶。我希望诸位也能好好地体会这句话。 既然如此， 莎士比亚说过一句富有哲理的话，抛弃时间的人，时间也抛弃他。这启发了我， 我们不得不面对一个非常尴尬的事实，那就是， 问题的关键究竟为何？ 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 总结的来说， 既然如何， 带着这些问题，我们来审视一下科学和人文谁更有意义。 每个人都不得不面对这些问题。 在面对这种问题时， 总结的来说， 问题的关键究竟为何？ 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 科学和人文谁更有意义因何而发生？ 科学和人文谁更有意义，到底应该如何实现。 科学和人文谁更有意义因何而发生？ 韩非在不经意间这样说过，内外相应，言行相称。这似乎解答了我的疑惑。 阿卜·日·法拉兹说过一句富有哲理的话，学问是异常珍贵的东西，从任何源泉吸收都不可耻。带着这句话，我们还要更加慎重的审视这个问题： 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 带着这些问题，我们来审视一下科学和人文谁更有意义。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 查尔斯·史考伯说过一句富有哲理的话，一个人几乎可以在任何他怀有无限热忱的事情上成功。 这不禁令我深思。 一般来讲，我们都必须务必慎重的考虑考虑。 带着这些问题，我们来审视一下科学和人文谁更有意义。 杰纳勒尔·乔治·S·巴顿在不经意间这样说过，接受挑战，就可以享受胜利的喜悦。这不禁令我深思。 科学和人文谁更有意义的发生，到底需要如何做到，不科学和人文谁更有意义的发生，又会如何产生。 可是，即使是这样，科学和人文谁更有意义的出现仍然代表了一定的意义。 从这个角度来看， 既然如此， 科学和人文谁更有意义，到底应该如何实现。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 这样看来， 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 我们不得不面对一个非常尴尬的事实，那就是， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 达尔文曾经说过，敢于浪费哪怕一个钟头时间的人，说明他还不懂得珍惜生命的全部价值。带着这句话，我们还要更加慎重的审视这个问题： 既然如此， 了解清楚科学和人文谁更有意义到底是一种怎么样的存在，是解决一切问题的关键。 每个人都不得不面对这些问题。 在面对这种问题时， 科学和人文谁更有意义因何而发生？ 杰纳勒尔·乔治·S·巴顿曾经说过，接受挑战，就可以享受胜利的喜悦。带着这句话，我们还要更加慎重的审视这个问题： 裴斯泰洛齐在不经意间这样说过，今天应做的事没有做，明天再早也是耽误了。我希望诸位也能好好地体会这句话。 我们都知道，只要有意义，那么就必须慎重考虑。 我认为， 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 海贝尔说过一句富有哲理的话，人生就是学校。在那里，与其说好的教师是幸福，不如说好的教师是不幸。这不禁令我深思。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 从这个角度来看， 每个人都不得不面对这些问题。 在面对这种问题时， 那么， 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 非洲曾经提到过，最灵繁的人也看不见自己的背脊。这句话语虽然很短，但令我浮想联翩。 佚名说过一句富有哲理的话，感激每一个新的挑战，因为它会锻造你的意志和品格。这句话语虽然很短，但令我浮想联翩。 莎士比亚在不经意间这样说过，抛弃时间的人，时间也抛弃他。这不禁令我深思。 杰纳勒尔·乔治·S·巴顿曾经提到过，接受挑战，就可以享受胜利的喜悦。这似乎解答了我的疑惑。 左拉曾经提到过，生活的道路一旦选定，就要勇敢地走到底，决不回头。这似乎解答了我的疑惑。 一般来说， 问题的关键究竟为何？ 经过上述讨论， 在这种困难的抉择下，本人思来想去，寝食难安。 这样看来， 而这些并不是完全重要，更加重要的问题是， 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 现在，解决科学和人文谁更有意义的问题，是非常非常重要的。 所以， 既然如何， 我认为， 我认为， 我们不得不面对一个非常尴尬的事实，那就是， 美华纳说过一句富有哲理的话，勿问成功的秘诀为何，且尽全力做你应该做的事吧。带着这句话，我们还要更加慎重的审视这个问题： 就我个人来说，科学和人文谁更有意义对我的意义，不能不说非常重大。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 本人也是经过了深思熟虑，在每个日日夜夜思考这个问题。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 科学和人文谁更有意义因何而发生。
　　一般来说， 裴斯泰洛齐曾经说过，今天应做的事没有做，明天再早也是耽误了。带着这句话，我们还要更加慎重的审视这个问题： 爱迪生曾经提到过，失败也是我需要的，它和成功对我一样有价值。带着这句话，我们还要更加慎重的审视这个问题： 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 经过上述讨论， 从这个角度来看， 总结的来说， 塞内加曾经提到过，勇气通往天堂，怯懦通往地狱。这启发了我， 我们不得不面对一个非常尴尬的事实，那就是， 那么。
　　生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 这种事实对本人来说意义重大，相信对这个世界也是有一定意义的。 一般来说， 杰纳勒尔·乔治·S·巴顿曾经说过，接受挑战，就可以享受胜利的喜悦。我希望诸位也能好好地体会这句话。 而这些并不是完全重要，更加重要的问题是， 科学和人文谁更有意义，发生了会如何，不发生又会如何。 科学和人文谁更有意义，发生了会如何，不发生又会如何。 要想清楚，科学和人文谁更有意义，到底是一种怎么样的存在。 所谓科学和人文谁更有意义，关键是科学和人文谁更有意义需要如何写。 生活中，若科学和人文谁更有意义出现了，我们就不得不考虑它出现了的事实。 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 我们一般认为，抓住了问题的关键，其他一切则会迎刃而解。 既然如此， 我认为， 对我个人而言，科学和人文谁更有意义不仅仅是一个重大的事件，还可能会改变我的人生。 科学和人文谁更有意义，到底应该如何实现。 科学和人文谁更有意义因何而发生？ 米歇潘曾经说过，生命是一条艰险的峡谷，只有勇敢的人才能通过。我希望诸位也能好好地体会这句话。 一般来讲，我们都必须务必慎重的考虑考虑。`;

// https://github.com/JIUYANGZH/BullshitGenerator-English/blob/master/bullshitgenerator_English/bullshitgenerator.py
var LARGE_TEXT_EN = `Thus, we have to figure out what kind of exist brexit isMoreover, for many years, people with brexit have been viewed as different. It is disappointing that, for me, the emergent rise of brexit may change my life. By contrasts, what could we do anyway to ahchieve brexit. Chiefly, rebecca West told us that, the trouble about man is twofold. He cannot learn truths which are too complicated; he forgets truths which are too simple. That solved my problem. It is important to note that If we think about it from a different point of view, for example, friedrich Nietzsche stated in his book that, i have forgotten my umbrella.  That aroused my imagination. Similarly, with these questions, let's take a closer look to brexit. Therefore, we generally say, if we grasped the key of the problem of brexit, everthing else will be easily solved. With respect to We have been thinking about brexit for a long time.Above all, under this inevitable circumstance situation, we have to solve brexit. It is important to note that As this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. Above all, does the government provide help for brexit? Besides, several studies of the effects of brexit on the human systems have failed to demonstrate these effects. Similarly, the evidence presented in this assignment has shown that, in particular, david Mamet concluded that, it's only words.. unless they're true. That inspired me. However, brexit has become increasingly evident among teens according to many scientists, however, as this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. In addition, for me, the emergent rise of brexit may change my life. On the other hand, brexit continues to be a relevant controversial issue in society today, as In comparison, what should we do to give rise to brexit, and what should we do to prevent brexit's happening. Alternatively, the problem lies in the difference of people raised by brexit, on the contrary, what is the key point of brexit? To look at another way, the evidence presented in this assignment has shown that, to look at another way, for brexit, the key point is how to express brexit. Furthermore, why does brexit happen? It is important to note that What is the key point of brexit? Chiefly, according to Orson Scott Card, eko brushed a tear from her eye, and Immo jeered at her, but father held up a hand. "Never mock a tender heart," he said. That solved my problem. To look at another way, if we think about it from a different point of view, alternatively, what is the key point of brexit? Furthermore, as this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. It is important to note that We have been thinking about brexit for a long time.Since that is so, for me, the emergent rise of brexit may change my life. Thus, we have to face an embarrassing situation, that is, in that case, now, solving brexit is becoming a more and more significant issue, in fact, several studies of the effects of brexit on the human systems have failed to demonstrate these effects. To look at another way, when facing this difficult choice of brexit, i rarely slept well. By contrasts, if we think about it from a different point of view, as in the following example, rebecca McClanahan argued that, the shortest distance between two points is always under construction. Which enlighten me. With respect to If brexit takes place in our daliy life, we have to consider its consequence. In that case, several studies of the effects of brexit on the human systems have failed to demonstrate these effects. Besides, as far as I know, on the other hand, in general, however, brexit continues to be a relevant controversial issue in society today, as Thus, under this inevitable circumstance situation, we have to solve brexit. It is disappointing that, as far as I know, it is disappointing that, the problem lies in the difference of people raised by brexit, similarly, after seeing this evidence, there is no way we can agree with brexit. In other words, we have been thinking about brexit for a long time.In comparison, several studies of the effects of brexit on the human systems have failed to demonstrate these effects. On the other hand, brexit has become increasingly evident among teens according to many scientists, alternatively, more current studies of people who take brexit heavely show no evidence of emotional damage. On the one hand, from my piont of view, brexit means a lot to me, and In particular, rebecca West told us that, the trouble about man is twofold. He cannot learn truths which are too complicated; he forgets truths which are too simple. Which enlighten me. In particular, according to Ljupka Cvetanova, smile and the world will laugh at you. That inspired me. In addition, several studies of the effects of brexit on the human systems have failed to demonstrate these effects. Namely, steve Wozniak told us that, never trust a computer you can't throw out a window. That solved my problem. Besides, why does brexit happen? Since that is so, for me, the emergent rise of brexit may change my life. On the one hand, in my opinion, in fact, what is the key point of brexit? Above all, when facing this difficult choice of brexit, i rarely slept well. Another way of viewing this is, modern vehement arguments against brexit alone become suspect. As in the following example, according to David Mamet, it's only words.. unless they're true. That solved my problem. Similarly, why does brexit happen? Another way of viewing this is, we have to face an embarrassing situation, that is, in that case, brexit continues to be a relevant controversial issue in society today, as By contrasts, in general, on the one hand, we generally say, if we grasped the key of the problem of brexit, everthing else will be easily solved. In fact, now, solving brexit is becoming a more and more significant issue, above all, with these questions, let's take a closer look to brexit. However, what is the key point of brexit? In comparison, brexit is a common condition among civilians in today’s society, so, thus, after seeing this evidence, there is no way we can agree with brexit. With respect to we have to face an embarrassing situation, that is, it is disappointing that, what could we do anyway to ahchieve brexit. It is important to note that When facing this difficult choice of brexit, i rarely slept well. In particular, kamand Kojouri showed us that, what you choose also chooses you. That solved my problem. By contrasts, some people may disagree with brexit, by contrasts, as this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. Since that is so, more current studies of people who take brexit heavely show no evidence of emotional damage. Alternatively, for brexit, the key point is how to express brexit. Since that is so, what if brexit happens, and what if brexit does not happen. Since that is so, every person has to face these problems caused by brexit, when facing these problems, in fact, more current studies of people who take brexit heavely show no evidence of emotional damage. To look at another way, we have to face an embarrassing situation, that is, in that case, the problem lies in the difference of people raised by brexit, similarly, is inevitable. In fact, for me, the emergent rise of brexit may change my life. In comparison, what should we do to give rise to brexit, and what should we do to prevent brexit's happening. It is important to note that As this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. It is important to note that brexit has become increasingly evident among teens according to many scientists, that is to say, james Thurber mentioned that, all human beings should try to learn before they die what they are running from, and to, and why. That solved my problem. Similarly, the consequece of brexit is of great significance to me, and to many other people. Chiefly, to quote from William Penn, all Excess is ill: but Drunkenness is of the worst Sort. It spoils Health, dismounts the Mind, and unmans Men: it reveals Secrets, is Quarrelsome, lascivious, impudent, dangerous and Mad. In fine, he that is drunk is not a Man: because he is so long void of Reason, that distinguishes a Man from a Beast. That inspired me. Therefore, with these questions, let's take a closer look to brexit. Since that is so, as far as I know, thus, for me, the emergent rise of brexit may change my life. Thus, for brexit, the key point is how to express brexit. Above all, now, solving brexit is becoming a more and more significant issue, in fact, in my opinion, furthermore, is inevitable. Namely, plato, symposium / Phaedrus stated in his book that, there is truth in wine and children. That inspired me. Above all, as this subject continues to be looked down on people must realize that brexit are becoming more common in today’s world. In fact, what is the key point of brexit? On the one hand, but these are not the most urgent issue, a more pressing issue about brexit is, alternatively, the problem lies in the difference of people raised by brexit, on the other hand, but these are not the most urgent issue, a more pressing issue about brexit is, in addition, the problem lies in the difference of people raised by brexit, thus, if brexit takes place in our daliy life, we have to consider its consequence. Another possibility is, to a certain extent, brexit is right. That is to say, james Thurber showed us that, all human beings should try to learn before they die what they are running from, and to, and why. Which enlighten me. In fact, in my opinion, namely, to quote from Comte de Lautréamont, we say sound things when we do not strive to say to say extraordinary ones. Which brought a new way of thinking it. `;